<?php
/**
 * Creates the text files used during module installation that create the
 * taxonomy terms and hierarchy. The output files are:
 *   terms.dat  : The HILCC hierarchy, contains a unique KEY and indicates
 *     parent-son relationships. This file is parsed in millennium_install()
 *     where the appropriate taxonomy_save_term() calls are done.
 *   ranges.dat : SQL INSERT commands to create a table that relates KEY to one
 *     or more LCC call number ranges. This file is just imported directly in
 *     millennium_install()
 *   hilcc_terms.pot : A POT (Portable Extractor Template) file for
 *     localization/translation of the created terms.
 */
function hilcc_generate_taxonomy_importfile() {

  // Data file
  $lines = file("hilcc_subjects_map_columbia.dat");
  
  foreach ($lines as $linenum => $line) {
    // $line = "Sequential number\tKEY\tClass1\tClass2\tCAT 1\tCAT 2\tCAT 3\tCAT 4\tCAT 5"
    // example: 1  1184  NA |0000.0000||  NA |9999.9990||  Arts, Architecture & Applied Arts  Architecture    
    
    #echo "\n [$linenum] ---------------\n";
        
    // Ignore comments
    if (strpos($line, "#") === 0) {
      #echo "\tSkipping comment\n";
      continue;
    }
    
    // Split line into fields, trim just in case
    $d = explode("\t", $line);
    foreach($d as $n =>$v) {
      $d[$n] = trim($v);
    }
    
    // Create $cat
    // Example: Business & Economics|Business|Commerce|Commerce (General)
    $tmp = $d;
    unset($tmp[0]);
    unset($tmp[1]);
    unset($tmp[2]);
    unset($tmp[3]);
    $cat = implode('|', $tmp);
    $cat = preg_replace('/\|+$/', '', $cat);
    
    $hilcc_id = trim($d[1]);
    
    $tid = $d[0];
    
    // Add range reference
    $output_ranges[] = sprintf("INSERT INTO {hilcc_ranges} (hilcc_id, start, end) VALUES ('%s', '%s', '%s')", 
      $hilcc_id, $d[2], $d[3] );
    
    #echo "$cat\n";
    
    if (isset($cat_to_hilcc[$cat])) {
      #echo "\tSkipping term creation...\n";
      continue;
    }
    
    // Create the tree
    $tmp = explode("|", $cat);
    $last_cat_part = $tmp[sizeof($tmp)-1];
    $current_cat = "";
    $prev_cat = "";
    $level = 0;
    
    #print_r($tmp);
    #echo "\n";
    foreach ($tmp as $cat_part) {
      $level++;
      echo str_repeat("*", $level) . " $cat_part\n";
      $current_cat .= ($current_cat != "" ? "|":"") . $cat_part;
      if (isset($part_tid[$level][$cat_part])) {
        $prev_cat = $current_cat;
        $prev_tid = $part_tid[$level][$cat_part];
        #echo " (existe, tid=$prev_tid)\n";
        continue;
      }
      
      #$parent_tid=$cat_to_tid[$prev_cat] + 0;
      
      // Add this
      $parent = 0;
      $output_po[$cat_part] = $cat_part;
      if ($prev_tid != 0 && $level > 1) {
        $parent = $prev_tid;
      } else {
        $parent = 0;
      }
      
      if ($cat_part == $last_cat_part) {
        $hilcc_id_real = $hilcc_id;
      } else {
        $hilcc_id_real = "";
      }
      
      // offset|name|parent|hilcc_id|hilcc_cat
      $output_terms[] = "$tid\t$cat_part\t$parent\t$hilcc_id_real\t$current_cat";
      echo "\t\t$tid\t$cat_part\t$parent\t$hilcc_id_real\t$current_cat\n";
            
      $prev_cat = $current_cat;
      $prev_tid = $tid;
      $part_tid[$level][$cat_part] = $tid;
      $cat_to_hilcc[$current_cat] = $hilcc_id_real;
      $tid++;
    }

  }
  
  /*
   * Print out queries with placeholders
   * Lines will be something like this:
   * INSERT INTO {term_data} (tid, name, vid) VALUES (!start_tid + 3, 'Agricultural Economics', !vid)
   * with !tid and !vid being placeholders that will be replaced with actual values when 
   * Drupal installs the module.
   */
  $ok = file_put_contents( "ranges.dat", implode("\n", $output_ranges) );
  $ok = file_put_contents( "terms.dat", implode("\n", $output_terms) );
  
  /*
   * Print out pot file, for future translation
   */
  ksort($output_po);
  $out[]="# \$Id\$\n#\n# LANGUAGE translation of HILCC\n# Copyright YEAR NAME <EMAIL@ADDRESS>\n#, fuzzy\nmsgid \"\"\nmsgstr \"\"\n";
  foreach($output_po as $term) {
    $out[]="msgid \"$term\"";
    $out[]="msgstr \"\"\n";
  }
  $ok = file_put_contents( "hilcc_terms.pot", implode("\n", $out) );
  
}
hilcc_generate_taxonomy_importfile();
?>